@@ -226,7 +226,6 @@
       ISO_14496_PART10_VIDEO = 0x1b,    /* ISO/IEC 14496-10 Video (MPEG-4 part 10/AVC, aka H.264) */
       STREAM_VIDEO_MPEG      = 0x80,
       STREAM_AUDIO_AC3       = 0x81,
-      STREAM_SPU_BITMAP_HDMV = 0x90,
 
       STREAM_VIDEO_VC1       = 0xea,    /* VC-1 Video */
 
@@ -237,6 +236,10 @@
       HDMV_AUDIO_85_DTS_HRA   = 0x85, /* DTS-HRA */
       HDMV_AUDIO_86_DTS_HD_MA = 0x86, /* DTS-HD Master audio */
 
+      HDMV_SPU_BITMAP      = 0x90,
+      HDMV_SPU_INTERACTIVE = 0x91,
+      HDMV_SPU_TEXT        = 0x92,
+
     } streamType;
 
 #define WRAP_THRESHOLD       270000
@@ -777,7 +780,7 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
     return 1;
   }
 
-  if (m->descriptor_tag == STREAM_SPU_BITMAP_HDMV) {
+  if (m->descriptor_tag == HDMV_SPU_BITMAP) {
     long payload_len = ((buf[4] << 8) | buf[5]) - header_len - 3;
 
     m->content = p;
@@ -815,12 +818,25 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
       m->type |= BUF_AUDIO_A52;
       return 1;
 
-    } else if (m->descriptor_tag == HDMV_AUDIO_86_DTS_HD_MA ) {
+    } else if (m->descriptor_tag == HDMV_AUDIO_82_DTS ||
+               m->descriptor_tag == HDMV_AUDIO_86_DTS_HD_MA ) {
       m->content = p;
       m->size = packet_len;
       m->type |= BUF_AUDIO_DTS;
       return 1;
 
+    } else if (m->descriptor_tag == HDMV_AUDIO_80_PCM) {
+
+      m->content = p + 4;
+      m->size    = packet_len - 4;
+      m->type   |= BUF_AUDIO_LPCM_BE;
+
+      m->buf->decoder_flags  |= BUF_FLAG_SPECIAL;
+      m->buf->decoder_info[1] = BUF_SPECIAL_LPCM_CONFIG;
+      m->buf->decoder_info[2] = (p[3]<<24) | (p[2]<<16) | (p[1]<<8) | p[0];
+
+      return 1;
+
     } else if (m->descriptor_tag == ISO_13818_PES_PRIVATE
 	     && p[0] == 0x20 && p[1] == 0x00) {
       /* DVBSUB */
@@ -968,7 +984,7 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
 	else if ( this->numPreview<5 )
 	  m->buf->decoder_flags=BUF_FLAG_PREVIEW;
 	else
-	  m->buf->decoder_flags=BUF_FLAG_FRAME_END;
+	  m->buf->decoder_flags |= BUF_FLAG_FRAME_END;
       }
       m->buf->pts = m->pts;
       m->buf->decoder_info[0] = 1;
@@ -1487,7 +1503,16 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
       }
       break;
 
-    case STREAM_SPU_BITMAP_HDMV:
+    case HDMV_SPU_INTERACTIVE:
+    case HDMV_SPU_TEXT:
+      if (this->hdmv > 0) {
+        printf("demux_ts: Skipping unsupported HDMV subtitle stream_type: 0x%.2x pid: 0x%.4x\n",
+               stream[0], pid);
+        break;
+      }
+      /* fall thru */
+
+    case HDMV_SPU_BITMAP:
       if (this->hdmv > 0) {
 	if (pid >= 0x1200 && pid < 0x1300) {
 	  /* HDMV Presentation Graphics / SPU */
@@ -1539,7 +1564,10 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
             demux_ts_get_reg_desc(this, &format_identifier,
                     stream + 5, stream_info_length);
             /* If no format identifier, assume A52 */
-            if ((format_identifier == 0x41432d33) || (format_identifier == 0)) {
+            if (( format_identifier == 0x41432d33) ||
+                ( format_identifier == 0) ||
+                ((format_identifier == 0x48444d56 || this->hdmv>0) && stream[0] == HDMV_AUDIO_80_PCM) /* BluRay PCM */) {
+
                 demux_ts_pes_new(this, this->media_num, pid, this->audio_fifo, stream[0]);
                 this->audio_tracks[this->audio_tracks_count].pid = pid;
                 this->audio_tracks[this->audio_tracks_count].media_index = this->media_num;
